//http://chrisheydrick.com/2012/06/17/how-to-read-serial-data-from-an-arduino-in-linux-with-c-part-3/

//compilation
// gcc -o arduino_read arduino_readv2.c -Ibstrlib bstrlib/bstrlib.c bstrlib/bstraux.c

#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* String function definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#include "bstrlib/bstrlib.h"
#include "bstrlib/bstrwrap.h" 


/* My Arduino is on /dev/ttyACM0 */
char *portname = "/dev/ttyACM0";
//char *portname = "/dev/ttyUSB0";
char buf[128];

// fonction issue du programme bstest.c qui permet d'extraire le contenu d'une bstring
// avec affichage de l'adresse et contenu

static bstring dumpOut[16];
static int rot = 0;

static char * dumpBstring (const struct tagbstring * b) {
	rot = (rot + 1) % (unsigned)16;
	if (dumpOut[rot] == NULL) {
		dumpOut[rot] = bfromcstr ("");
		if (dumpOut[rot] == NULL) return "FATAL INTERNAL ERROR";
	}
	dumpOut[rot]->slen = 0;
	if (b == NULL) {
		bcatcstr (dumpOut[rot], "NULL");
	} else {
		static char msg[256];
		sprintf (msg, "%p", (void *)b);
		bcatcstr (dumpOut[rot], msg);

		if (b->slen < 0) {
			sprintf (msg, ":[err:slen=%d<0]", b->slen);
			bcatcstr (dumpOut[rot], msg);
		} else {
			if (b->mlen > 0 && b->mlen < b->slen) {
				sprintf (msg, ":[err:mlen=%d<slen=%d]", b->mlen, b->slen);
				bcatcstr (dumpOut[rot], msg);
			} else {
				if (b->mlen == -1) {
					bcatcstr (dumpOut[rot], "[p]");
				} else if (b->mlen < 0) {
					bcatcstr (dumpOut[rot], "[c]");
				}
				bcatcstr (dumpOut[rot], ":");
				if (b->data == NULL) {
					bcatcstr (dumpOut[rot], "[err:data=NULL]");
				} else {
					bcatcstr (dumpOut[rot], "\"");
					bcatcstr (dumpOut[rot], (const char *) b->data);
					bcatcstr (dumpOut[rot], "\"");
				}
			}
		}
	}
	return (char *) dumpOut[rot]->data;
}

int main(int argc, char *argv[])
{
 int fd;
 float JoyData[13]; //JoyX, JoyXBar, JoyY, JoyYBar;
 float test;
/* Open the file descriptor in non-blocking mode */
 fd = open(portname, O_RDWR | O_NOCTTY);
 printf("fd opened as %i\n", fd);
 
/* Set up the control structure */
 struct termios toptions;
 
 /* Get currently set options for the tty */
 //tcgetattr(fd, &toptions);
 
 
/* Set custom options */
 
/* 115200 baud */
 //cfsetispeed(&toptions, B115200);
 //cfsetospeed(&toptions, B115200);
 /* 8 bits, no parity, no stop bits */
 //toptions.c_cflag &= ~PARENB;
 //toptions.c_cflag &= ~CSTOPB;
 //toptions.c_cflag &= ~CSIZE;
 //toptions.c_cflag |= CS8;
 /* no hardware flow control */
 //toptions.c_cflag &= ~CRTSCTS;
 /* enable receiver, ignore status lines */
 //toptions.c_cflag |= CREAD | CLOCAL;
 /* disable input/output flow control, disable restart chars */
 //toptions.c_iflag &= ~(IXON | IXOFF | IXANY);
 /* disable canonical input, disable echo,
 disable visually erase chars,
 disable terminal-generated signals */
 //toptions.c_iflag &= ~(ICANON | ECHO | ECHOE | ISIG);
 /* disable output processing */
 //toptions.c_oflag &= ~OPOST;
 
 /* get current serial port settings */
  tcgetattr(fd, &toptions);
  /* set 115200 baud both ways */
  cfsetispeed(&toptions, B115200);
  cfsetospeed(&toptions, B115200);
  /* 8 bits, no parity, no stop bits */
  toptions.c_cflag &= ~PARENB;
  toptions.c_cflag &= ~CSTOPB;
  toptions.c_cflag &= ~CSIZE;
  toptions.c_cflag |= CS8;
  /* Canonical mode */
  toptions.c_lflag |= ICANON;
  /* commit the serial port settings */
  tcsetattr(fd, TCSANOW, &toptions);
 
 
 //https://github.com/cheydrick/Canonical-Arduino-Read/blob/master/canonicalarduinoread.c
 /* Canonical mode */
  toptions.c_lflag |= ICANON;
 
/* wait for 24 characters to come in before read returns */
 toptions.c_cc[VMIN] = 70;
 /* no minimum time to wait before read returns */
 toptions.c_cc[VTIME] = 0;
 
/* commit the options */
 tcsetattr(fd, TCSANOW, &toptions);
 
/* Wait for the Arduino to reset */
//http://playground.arduino.cc/Main/DisablingAutoResetOnSerialConnection
printf("Waiting for arduino to reset...\n");
//usleep(3500000);
usleep(3500000);
/* Flush anything already in the serial buffer */
tcflush(fd, TCIFLUSH);

//define the field separator in the buffer, and decimal point for float conversion
bstring separator = bfromcstr(";");
bstring decimalpoint = bfromcstr(".");
bstring src;
//définition des bstrList utiles au traitement
struct bstrList * sublist;
int i;	
	
 while(1)
 {
	 /* Send byte to trigger Arduino to send string back */
	write(fd, "0", 1);
	/* read up to 128 bytes from the fd */
	int n = read(fd, buf, 70);
	buf[n] = 0;
	//tcflush(fd, TCIFLUSH);
	/* print how many bytes read */
	//printf("%i bytes got read...\n", n);
	/* print what's in the buffer */
	if (n>1)
	{
	//printf("%i, %s",n, buf);
	
	//starting data conversion
	src = bfromcstr(buf);
	if (!src)
	{
		fprintf (stderr, "Out of memory");
	} 
	else 
	{
	//fprintf(stderr,"Data src : \"%s\"\n\n",(char *) src->data);
	}
	//premier éclatement suivant les ";"
	sublist = bsplitstr(src, separator);
	if (sublist) 
	{
		for (i=0; i < sublist->qty; i++)
			{
			//printf ("Dump Data %d : %s\n", i, dumpBstring(sublist->entry[i]));
			//retire les espaces
			btrimws (sublist->entry[i]);
			//conversion en double
			
			sscanf((const char *)sublist->entry[i]->data,"%f",&test);
			JoyData[i]=test;
			//sscanf( (const char *) sublist->entry[i]->data,"%f",JoyData+i);
			//printf("Dump String Element %d : %s\n", i,dumpBstring(sublist->entry[i]));
			//printf("Conversion Text %s -> Number %lf\n",(const char *) sublist->entry[i]->data,test);
		}
		printf("%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f\n",JoyData[0],JoyData[1],JoyData[2],JoyData[3],JoyData[4],JoyData[5],JoyData[6],JoyData[7],JoyData[8],JoyData[9],JoyData[10],JoyData[11],JoyData[12],JoyData[13]);
		
	}
    }
}
return 0;
}
