#include <stdio.h>
#include "bstrlib.h"

/*
 *  In the usenet newsgroups comp.lang.c Gary Morris wrote:
 *
 *    What I have to do is parse a JavaScript file that
 *  will ALWAYS have the following format:
 *  
 *  ************************************************
 *  <!-- 
 *  document.writeln('<TABLE BORDER="0" CELLPADDING="2">');
 *  document.writeln('  <TR>');
 *  ...(many more lines all starting with "document.writeln('" )
 *  document.writeln('  </TR>');
 *  document.writeln('</TABLE>');
 *  // -->
 *  ************************************************
 *    It is auto-generated by another web site, and my job is to
 *  get it to plain HTML format so that it can be linked to in an
 *  email.  What we want is for the end result to be:
 *  
 *  ************************************************
 *  
 *  <TABLE BORDER="0" CELLPADDING="2">
 *  <TR>
 *  ...(many more lines no longer with the "document.writeln('" )
 *  </TR>
 *  </TABLE>
 *  
 *  ************************************************
 */

static int parseLines (const bstring src) {
struct tagbstring token0 = bsStatic ("document.writeln('");
struct tagbstring token1 = bsStatic ("');");
struct tagbstring t, u;
int i, j;

	/* Reference to where 1st token might match in src string */
	blk2tbstr (t, src->data, token0.slen);
	for (i=0; i < src->slen - token0.slen; i++) {

		/* Does the 1st token match exactly? */
		if (biseq (&t, &token0)) {

			/* Reference to where 2nd token might match */
			blk2tbstr (u, t.data, token1.slen);
			for (j = i; j < src->slen - token1.slen; j++) {

				/* Does the 2nd token match exactly? */
				if (biseq (&u, &token1)) {

					/* Construct middle string */
					bstring b = blk2bstr (t.data + token0.slen, j - i - token0.slen);

					/* Output the '\0' terminated buffer */
					puts ((char *) b->data);
					bdestroy (b);
					break;
				}

				/* Shift 2nd token scan downward */
				u.data++;
			}
		}

		/* Shift 1st token scan downward */
		t.data++;
	}
	return 0;
}

int main (int argc, char * argv[]) {
FILE * fp;

	if (argc < 2) {
		printf ("%s [inputfile]\n", argv[0]);
		return -__LINE__;
	}

	if (NULL != (fp = fopen (argv[1], "rb"))) {
		bstring src = bread ((bNread) fread, fp);
		int ret = parseLines (src);
		fclose (fp);
		bdestroy (src);
		return ret;
	}
	return -__LINE__;
}

